<?php
class ControllerExtensionPaymentStripe extends Controller {
  private $error = array();

  public function index() {
    $this->load->language('extension/payment/stripe');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->load->model('setting/setting');

    if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
      $this->model_setting_setting->editSetting('payment_stripe', $this->request->post);

      $this->session->data['success'] = $this->language->get('text_success');

      $this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));
    }

    $data['heading_title']      = $this->language->get('heading_title');

    $data['text_edit']          = $this->language->get('text_edit');
    $data['text_enabled']       = $this->language->get('text_enabled');
    $data['text_disabled']      = $this->language->get('text_disabled');
    $data['text_all_zones']     = $this->language->get('text_all_zones');
    $data['text_test']          = $this->language->get('text_test');
    $data['text_live']          = $this->language->get('text_live');
    $data['text_authorization'] = $this->language->get('text_authorization');
    $data['text_charge']        = $this->language->get('text_charge');

    $data['entry_public']       = $this->language->get('entry_public');
    $data['entry_key']          = $this->language->get('entry_key');
    $data['entry_method']       = $this->language->get('entry_method');
    $data['entry_total']        = $this->language->get('entry_total');
    $data['help_total']         = $this->language->get('help_total');
    $data['entry_order_status'] = $this->language->get('entry_order_status');
    $data['entry_geo_zone']     = $this->language->get('entry_geo_zone');
    $data['entry_status']       = $this->language->get('entry_status');
    $data['entry_sort_order']   = $this->language->get('entry_sort_order');

    $data['button_save']        = $this->language->get('button_save');
    $data['button_cancel']      = $this->language->get('button_cancel');

    if (isset($this->error['warning'])) {
      $data['error_warning'] = $this->error['warning'];
    } else {
      $data['error_warning'] = '';
    }

    if (isset($this->error['license'])) {
      $data['error_license'] = $this->error['license'];
    } else {
      $data['error_license'] = '';
    }

    if (isset($this->error['public'])) {
      $data['error_public'] = $this->error['public'];
    } else {
      $data['error_public'] = '';
    }

    if (isset($this->error['key'])) {
      $data['error_key'] = $this->error['key'];
    } else {
      $data['error_key'] = '';
    }

    $data['breadcrumbs'] = array();

    $data['breadcrumbs'][] = array(
        'text'      => $this->language->get('text_home'),
        'href'      => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
    );

    $data['breadcrumbs'][] = array(
        'text'      => $this->language->get('text_extension'),
        'href'      => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
    );

    $data['breadcrumbs'][] = array(
        'text'      => $this->language->get('heading_title'),
        'href'      => $this->url->link('extension/payment/stripe', 'user_token=' . $this->session->data['user_token'], true)
    );

    $data['action'] = $this->url->link('extension/payment/stripe', 'user_token=' . $this->session->data['user_token'], true);

    $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);

    if (isset($this->request->post['payment_stripe_public_key'])) {
      $data['payment_stripe_public_key'] = $this->request->post['payment_stripe_public_key'];
    } else {
      $data['payment_stripe_public_key'] = $this->config->get('payment_stripe_public_key');
    }

    if (isset($this->request->post['payment_stripe_private_key'])) {
      $data['payment_stripe_private_key'] = $this->request->post['payment_stripe_private_key'];
    } else {
      $data['payment_stripe_private_key'] = $this->config->get('payment_stripe_private_key');
    }

    if (isset($this->request->post['stripe_mode'])) {
      $data['stripe_mode'] = $this->request->post['stripe_mode'];
    } else {
      $data['stripe_mode'] = $this->config->get('stripe_mode');
    }

    if (isset($this->request->post['payment_stripe_method'])) {
      $data['payment_stripe_method'] = $this->request->post['payment_stripe_method'];
    } else {
      $data['payment_stripe_method'] = $this->config->get('payment_stripe_method');
    }

    if (isset($this->request->post['payment_stripe_order_status_id'])) {
      $data['payment_stripe_order_status_id'] = $this->request->post['payment_stripe_order_status_id'];
    } else {
      $data['payment_stripe_order_status_id'] = $this->config->get('payment_stripe_order_status_id');
    }

    $this->load->model('localisation/order_status');

    $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

    if (isset($this->request->post['payment_stripe_geo_zone_id'])) {
      $data['payment_stripe_geo_zone_id'] = $this->request->post['payment_stripe_geo_zone_id'];
    } else {
      $data['payment_stripe_geo_zone_id'] = $this->config->get('payment_stripe_geo_zone_id');
    }

    $this->load->model('localisation/geo_zone');

    $data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

    if (isset($this->request->post['payment_stripe_status'])) {
      $data['payment_stripe_status'] = $this->request->post['payment_stripe_status'];
    } else {
      $data['payment_stripe_status'] = $this->config->get('payment_stripe_status');
    }

    if (isset($this->request->post['payment_stripe_paymentbutton'])) {
      $data['payment_stripe_paymentbutton'] = $this->request->post['payment_stripe_paymentbutton'];
    } else {
      $data['payment_stripe_paymentbutton'] = $this->config->get('payment_stripe_paymentbutton');
    }
    if ($this->request->server['HTTPS']) {
      $b = HTTPS_SERVER;
    } else {
      $b = HTTP_SERVER;
    }
    $data['is610'] = $this->is610($b);


    if (isset($this->request->post['payment_stripe_total'])) {
      $data['payment_stripe_total'] = $this->request->post['payment_stripe_total'];
    } else {
      $data['payment_stripe_total'] = $this->config->get('payment_stripe_total');
    }

    if (isset($this->request->post['payment_stripe_payment_name'])) {
      $data['payment_stripe_payment_name'] = $this->request->post['payment_stripe_payment_name'];
    } else {
      $data['payment_stripe_payment_name'] = $this->config->get('payment_stripe_payment_name');
    }

    if (isset($this->request->post['payment_stripe_sort_order'])) {
      $data['payment_stripe_sort_order'] = $this->request->post['payment_stripe_sort_order'];
    } else {
      $data['payment_stripe_sort_order'] = $this->config->get('payment_stripe_sort_order');
    }

    if (isset($this->request->post['payment_stripe_license_key'])) {
      $data['payment_stripe_license_key'] = $this->request->post['payment_stripe_license_key'];
    } else {
      $data['payment_stripe_license_key'] = $this->config->get('payment_stripe_license_key');
    }

    $data['b'] = $b;

    $data['header'] = $this->load->controller('common/header');
    $data['column_left'] = $this->load->controller('common/column_left');
    $data['footer'] = $this->load->controller('common/footer');

    $this->response->setOutput($this->load->view('extension/payment/stripe', $data));
  }

  private function is610($b) {
    $url = 'https://license.stripe-opencart.com/post610';
    $ch = curl_init($url);
    $postString = http_build_query(array('b'=>$b), '', '&');
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $postString);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($ch);
    curl_close($ch);
    $sessiondata = json_decode($response);
    return $sessiondata->{'success'};
  }


  protected function validate() {
    $this->load->language('extension/payment/stripe');

    if (!$this->user->hasPermission('modify', 'extension/payment/stripe')) {
      $this->error['warning'] = $this->language->get('error_permission');
    }

    if ($this->request->server['HTTPS']) {
      $b = HTTPS_SERVER;
    } else {
      $b = HTTP_SERVER;
    }
    if($this->is610($b)) {
      return !$this->error;
    }

    if (!$this->request->post['payment_stripe_license_key']) {
      $this->error['license'] = $this->language->get('error_msg_license');
    } else {
      $cur = $this->request->post['payment_stripe_license_key'];
      $prev = $this->config->get('payment_stripe_license_key');

      if ($cur == $prev) {
        $ver = $this->language->get('version');
        $url = 'https://license.stripe-opencart.com/verify?key=' . $cur . '&extension=' . 'oc_stripe_basic&v=' . $ver . '&b=' . $b;;

        if (ini_get('allow_url_fopen')) {
            $arrContextOptions=array(
                "ssl"=>array(
                    "verify_peer"=>false,
                    "verify_peer_name"=>false,
                ),
            );
            $result = file_get_contents($url, false, stream_context_create($arrContextOptions));
        } else {
          $ch = curl_init();
          curl_setopt($ch, CURLOPT_URL, $url);
          curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
          $result = curl_exec($ch);
          curl_close($ch);
        }
        $json = json_decode($result);

        if (!$json->{'success'}) {
          if ($json->{'type'} == 'expired') {
            $this->error['license'] = $this->language->get('error_license_expired');
          } else {
            $this->error['license'] = $this->language->get('error_msg_license');
          }
        }
      } else {
        if ($this->request->post['payment_stripe_license_is_activated'] == 0) {
          $this->error['license'] = $this->language->get('error_activate_license');
        }
      }
    }

    return !$this->error;
  }
}
