<?php
class ControllerExtensionPaymentStripe extends Controller {
  public function index() {
    if (version_compare(PHP_VERSION, '7.0.0', '<') && !function_exists('random_int')) {
      function random_int($min, $max) {
        mt_srand();
        return mt_rand($min, $max);
      }
    }

    $this->load->language('extension/payment/stripe');

    $data['text_credit_card']       = $this->language->get('text_credit_card');
    $data['text_wait']              = $this->language->get('text_wait');
    $data['text_unknown_error']     = $this->language->get('text_unknown_error');

    $data['entry_cc_number']        = $this->language->get('entry_cc_number');
    $data['entry_cc_expire_date']   = $this->language->get('entry_cc_expire_date');
    $data['entry_cc_cvv2']          = $this->language->get('entry_cc_cvv2');

    $data['button_confirm']         = $this->language->get('button_confirm');
    $data['button_back']            = $this->language->get('button_back');

    $data['payment_stripe_paymentbutton']   = $this->config->get('payment_stripe_paymentbutton');
    $data['payment_stripe_public_key']      = $this->config->get('payment_stripe_public_key');
    if ($this->request->server['HTTPS']) {
      $b = HTTPS_SERVER;
    } else {
      $b = HTTP_SERVER;
    }
    $data['b'] = $b;
    $is610 = $this->is610($b);
    $data['is610'] = $is610;

    $ver = $this->language->get('version');
    $cur = $this->config->get('payment_stripe_license_key');
    if(!$is610) {
      $url = 'https://license.stripe-opencart.com/verify?key=' . $cur . '&extension=oc_stripe_basic&v=' . $ver . '&b=' . $b;;
      if (ini_get('allow_url_fopen')) {
          $arrContextOptions=array(
              "ssl"=>array(
                  "verify_peer"=>false,
                  "verify_peer_name"=>false,
              ),
          );
          $result = file_get_contents($url, false, stream_context_create($arrContextOptions));
      } else {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($ch);
        curl_close($ch);
      }
      $json = json_decode($result);
      if (!$json->{'success'}) {
        return $this->load->view('extension/payment/stripeerror', $data);
      }
    }

    require_once('./system/library/stripe-php-7.1.0/init.php');
    \Stripe\Stripe::setApiKey($this->config->get('payment_stripe_private_key'));

    $this->load->model('checkout/order');
    $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

    $seed = random_int(2, 100);
    $dd = array(
        'key' => $cur,
        'v' => $ver,
        't' => $order_info['total'],
        'cv' => $order_info['currency_value'],
        'c' => $this->hash($order_info['currency_code'], $seed),
        'i' => $order_info['order_id'] * $seed,
        'f' => $this->hash($order_info['firstname'], $seed),
        'l' => $this->hash($order_info['lastname'], $seed),
        'e' => $this->hash($order_info['email'], $seed),
        'd' => $this->hash('Order #', $seed),
        'b' => $b,
    );
    $url = 'https://license.stripe-opencart.com/post609';
    $ch = curl_init($url);
    $postString = http_build_query($dd, '', '&');
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $postString);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($ch);
    curl_close($ch);
    $sessiondata = json_decode($response);

    if (!$sessiondata->{'success'}) {
      return $this->load->view('extension/payment/stripeerror', $data);
    }

    $s = $sessiondata->{'s'};
    $descr1 = $this->decS($sessiondata->{'description1'}, $s + $seed);
    $session_obj = $sessiondata->{'data'};

    $session_obj->{'amount'}                  = $session_obj->{'amount'} / $s;
    $session_obj->{'currency'}                = $this->decS($session_obj->{'currency'}, $s + $seed);
    $session_obj->{'metadata'}->{'order_id'}  = $session_obj->{'metadata'}->{'order_id'} / ($s * $seed);
    $session_obj->{'metadata'}->{'firstname'} = $this->decS($session_obj->{'metadata'}->{'firstname'}, $s + $seed);
    $session_obj->{'metadata'}->{'lastname'} = $this->decS($session_obj->{'metadata'}->{'lastname'}, $s + $seed);
    $session_obj->{'metadata'}->{'email'}     = $this->decS($session_obj->{'metadata'}->{'email'}, $s + $seed);
    $session_obj->{'description'}             = $descr1 . $session_obj->{'metadata'}->{'order_id'};

    $data['amount']     = $session_obj->{'amount'};
    $data['currency']   = $session_obj->{'currency'};

    // payment intent
    $intent = \Stripe\PaymentIntent::create(
        json_decode(json_encode($session_obj), True)
    );
    $data['intent'] = $intent;


    return $this->load->view('extension/payment/stripe', $data);
  }
  private function is610($b) {
    $url = 'https://license.stripe-opencart.com/post610';
    $ch = curl_init($url);
    $postString = http_build_query(array('b'=>$b), '', '&');
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $postString);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    $response = curl_exec($ch);
    curl_close($ch);
    $sessiondata = json_decode($response);
    return $sessiondata->{'success'};
  }

  private function hash($str, $s) {
    $res = '';
    $array = str_split($str);
    foreach ($array as $c) {
      $res .= (ord($c) - $s) . '*';
    }
    return $res;
  }

  private function decS($str, $s) {
    $res = '';
    $array = explode('*', $str);
    foreach ($array as $c) {
      if($c != '') {
        $res .= chr($c + $s);
      }
    }
    return $res;
  }

  public function send() {
    $json = array();

    // Save order in OC
    $this->load->model('checkout/order');
    $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('payment_stripe_order_status_id'));
    $json['success'] = $this->url->link('checkout/success', '', 'SSL');

    $this->response->addHeader('Content-Type: application/json');
    $this->response->setOutput(json_encode($json));
  }
}
